@@ -34,7 +34,7 @@ class StrictHeaderParsingFailure(RootException, ValueError):
     See :func:`ietfparse.headers.parse_forwarded` for an example.
 
     """
-    def __init__(self, header_name, header_value):
+    def __init__(self, header_name: str, header_value: str) -> None:
         super(StrictHeaderParsingFailure,
               self).__init__(header_name, header_value)
         self.header_name = header_name
